//
// client.cpp
// ~~~~~~~~~~
//
// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#include <boost/asio.hpp>
#include <boost/log/attributes.hpp>
#include <boost/log/core.hpp>
#include <boost/log/utility/setup/common_attributes.hpp>
// #include <boost/log/sinks/text_file_backend.hpp>
#include <boost/log/trivial.hpp>
#include <boost/log/utility/setup/file.hpp>
#include <cstdlib>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <map>
#include <string>

#include "config.h"
#include "session.h"

// #include <boost/date_time/posix_time/posix_time_types.hpp>
#include <boost/log/expressions.hpp>
#include <boost/log/support/date_time.hpp>

/*
boost log linking -
undefined reference to `void boost::log::v2_mt_posix::init_from_stream
https://github.com/boostorg/log/issues/46
 */

/*
BOOST_LOG_ATTRIBUTE_KEYWORD(scope, "Scope",
                            boost::log::attributes::named_scope::value_type)
*/

void init_logging(void) {
  // because TimeStamp is missing by default.
  boost::log::add_common_attributes();

  /*
  auto fmtScope = boost::log::expressions::format_named_scope(
      "Scope", boost::log::keywords::format = "%n(%f:%l)",
      boost::log::keywords::iteration = boost::log::expressions::reverse,
      boost::log::keywords::depth = 2);
  */

  boost::log::add_file_log(
      // This gives dated logfile name.
      boost::log::keywords::file_name = "proxy-%Y-%m-%d.log",
      // This appends to the logfile (instead of overwrite)
      boost::log::keywords::open_mode = std::ios_base::out | std::ios_base::app,
      boost::log::keywords::auto_flush = true,

      // boost::log::keywords::format = "[%TimeStamp%] %Severity%  : %Message%"
      boost::log::keywords::format =
          (boost::log::expressions::stream
           << boost::log::expressions::format_date_time<
                  boost::posix_time::ptime>("TimeStamp", "%H:%M:%S.%f %I:%M %p")
           // boost::posix_time::ptime>("TimeStamp", "%H:%M:%S.%f")
           << " " << std::setw(8) << boost::log::trivial::severity << " "
           << boost::log::expressions::smessage)

  );

  /*
    auto core = boost::log::core::get();
    // boost::log::core::get()->set_filter(boost::log::trivial::severity >=
    //                                     boost::log::trivial::info);

    // core->set_filter(boost::log::trivial::severity >=
    // boost::log::trivial::info);

    core->set_filter(boost::log::trivial::severity >= logging_level);

    // core->add_global_attribute("Scopes", boost::log::keywords::format = "%n
    // (%f:%l)"); core->set_formatter("Scopes", boost::log::keywords::format =
    "%n
    // (%f:%l)"); logging::core::get()->add_global_attribute("Scopes",
    // attributes::named_scope());

    // core->add_global_attribute("Scope",
    boost::log::attributes::named_scope());
  */
}

int main(int argc, char *argv[]) {
  // boost::json::json_value config;

  if (argc != 2) {
    std::cerr << "Usage: twproxy <filename>" << std::endl;
    return EXIT_FAILURE;
  }

  std::map<std::string, std::string> config = yaml_parse(argv[1]);

  auto value = config.find("log_level");
  int log_level = 2;
  if (value != config.end()) {
    log_level = std::stoi(value->second); // config[value]);
  }

  init_logging();

  auto core = boost::log::core::get();
  core->set_filter(boost::log::trivial::severity >= log_level);

  BUGZ_LOG(error) << "Logging level:" << log_level;

  /*
  try {
    // Parse the file as JSON
    config = yaml_parse( argv[1] );

  } catch (std::exception const &e) {
    std::cerr << "Caught exception: " << e.what() << std::endl;
    return EXIT_FAILURE;
  }
  */

  bool config_ok = true;

  // for (const char *key : {"server", "host", "port"}) {
  for (auto key : {"server", "host", "port"}) {
    auto pos = config.find(key);
    if (pos == config.end()) {
      config_ok = false;
      std::cout << "Config file missing: " << key << std::endl;
      BOOST_LOG_TRIVIAL(fatal) << "Config file missing: " << key;
    }
    BOOST_LOG_TRIVIAL(info) << "Config: " << key << " : " << config[key];
  }
  if (!config_ok)
    return 2;

  int port = std::stoi(config["server"]);
  // int port = 9999; // 2002;

  try {

    boost::asio::io_service io_service;

    boost::asio::ip::tcp::endpoint endpoint(boost::asio::ip::tcp::v4(),
                                            port); // std::atoi(argv[i]));
    // connect to the BBS
    server s(io_service, endpoint, config["host"], config["port"]);
    //"127.0.0.1", "2023");

    io_service.run();
  } catch (std::exception &e) {
    std::cerr << "Exception: " << e.what() << "\n";
  }

  return EXIT_SUCCESS;
}
